@@ -21,6 +21,8 @@ import shutil
 from jinja2 import Template
 from copy import deepcopy
 import requests
+import time, datetime
+import pytest
 from cvapianalyser import CommunityEdition
 from openapispecdiff import OpenApiSpecDiff
 
@@ -36,12 +38,11 @@ HTTP_RESPONSE_CODES = ["100", "101", "200", "201", "202", "203", "204", "205", "
 
 root = os.path.dirname(os.path.abspath(__file__))
 templates_dir = os.path.join(root, 'templates')
-fuzz_words_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'wfuzz/wordlist')
-fuzz_types = ["general", "injections", "vulns", "webservices", "stress", "others"]
+fuzz_words_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'fuzzdb/attack')
+fuzz_types = ["fuzzdb/attack"]#os.listdir(os.path.join(root, "fuzzdb/attack"))#["general", "injections", "vulns", "webservices", "stress", "others"]
 ANOMOLY_THRESHOLD = 1.07142
 RESERVED_PARAMETERS = ["$RESPONSE_ERROR", "$CV_EVENT_ID"]
 
-
 def update_nested_dict(key, value, dictionary):
     for k, v in dictionary.items():
         if k == key:
@@ -126,9 +127,9 @@ class CloudvectorDAST(object):
         if not os.path.exists("tests"):
             os.mkdir("tests")
         if do_fuzz:
-            if not os.path.exists((os.path.join(os.getcwd(), "wfuzz"))):
-                shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), "wfuzz"),
-                                os.path.join(os.getcwd(), "wfuzz"))
+            if not os.path.exists((os.path.join(os.getcwd(), "fuzzdb"))):
+                shutil.copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), "fuzzdb"),
+                                os.path.join(os.getcwd(), "fuzzdb"))
         self.params_captured_in_traffic = {}
         self.prepped_spec = []
         if ".txt" not in self.apispec_two_path:
@@ -159,7 +160,8 @@ class CloudvectorDAST(object):
         with open("tests/params_captured.json", "w+") as fobj:
             json.dump(self.params_captured_in_traffic, fobj, indent=4)
         self.create_pyfixtures()
-        self.create_pytest_methods(cv_events, custom_validations)
+        if ce_host:
+            self.create_pytest_methods(cv_events, custom_validations)
         if do_fuzz:
             self.create_fuzz_test_methods(cv_events)
         self.create_assertions(fuzzing=do_fuzz)
@@ -182,6 +184,7 @@ class CloudvectorDAST(object):
 
     def _prepare_spec_for_test_generation(self, input_spec, specific_apis=None):
         parsed_new_spec = self._get_spec_parsed(input_spec)
+        params_info_from_spec = {}
         apis_from_spec = []
         if "servers" in parsed_new_spec:
             from urllib.parse import urlparse
@@ -200,7 +203,11 @@ class CloudvectorDAST(object):
                     continue
             if api not in self.respcodes_from_spec:
                 self.respcodes_from_spec[api] = {}
+            if api not in params_info_from_spec:
+                params_info_from_spec[api] = {}
             for method, minfo in info.items():
+                if method not in params_info_from_spec[api]:
+                    params_info_from_spec[api][method]={"required":[], "optional":[]}
                 x = {}
                 x["method"] = method.lower()
                 x["host"] = host
@@ -225,13 +232,20 @@ class CloudvectorDAST(object):
                 self.respcodes_from_spec[x["http-req-url"]][method] = list(minfo.get("responses").keys())
                 x["url"] = x["url"].replace("{", "{{").replace("}", "}}")
                 for _ in minfo.get("parameters", []):
+                    if _.get("required", True):
+                        if type(_.get("required")) is bool:
+                            params_info_from_spec[api][method]["required"].append(_.get("name"))
+                        else:
+                            params_info_from_spec[api][method]["required"]+=_.get("required",[])
+                    else:
+                        params_info_from_spec[api][method]["optional"].append(_.get("name"))
                     if _.get("in") == "header":
                         x["header"][_.get("name")] = ""
                     elif _.get("in") == "query":
                         if "?" not in x["url"]:
-                            x["url"] += "?" + _.get("name") + "={{" + _.get("name") + "}}"
+                            x["url"] += "?" + _.get("name") + "={{" + _.get("name").replace(".", "_") + "}}"
                         else:
-                            x["url"] += "&" + _.get("name") + "={{" + _.get("name") + "}}"
+                            x["url"] += "&" + _.get("name") + "={{" + _.get("name").replace(".", "_") + "}}"
                         x["params"].append(_.get("name"))
                     else:
                         if _.get("name"):
@@ -243,6 +257,10 @@ class CloudvectorDAST(object):
                     self.params_from_spec[api] = {}
                 self.params_from_spec[api][method] = {_: [] for _ in x["params"]}
                 apis_from_spec.append([x, None])
+        if not os.path.exists("tests/data"):
+            os.mkdir("tests/data")
+        with open("tests/data/params_info_from_spec.json", "a+") as _file:
+            json.dump(params_info_from_spec, _file)
         return apis_from_spec
 
     def _parse_cv_config(self, path):
@@ -306,7 +324,7 @@ class CloudvectorDAST(object):
 
     def _get_fuzz_details(self, fuzz_type):
         fuzz_values = []
-        fuzz_values.append({"path": os.path.join("wfuzz/wordlist/", fuzz_type),
+        fuzz_values.append({"path": fuzz_type,
                             "fuzz_type": fuzz_type})
         return fuzz_values
 
@@ -430,7 +448,8 @@ class CloudvectorDAST(object):
         assertions_map = {}
         if url not in self.assertions_map:
             url = \
-            url.replace("//", "/").replace("/", "__").replace("{", "").replace("}", "").replace(" ", "").split("?")[0]
+                url.replace("//", "/").replace("/", "__").replace("{", "").replace("}", "").replace(" ", "").split("?")[
+                    0]
             if url.startswith("__"):
                 url = "_" + url[2:]
             self.assertions_map[url] = {}
@@ -467,7 +486,7 @@ class CloudvectorDAST(object):
                             validations.append([str(param) + "_invalid", str(param) + "_arg = 'iamdummy'", to_check])
         return validations
 
-    def _process_cv_events(self,cv_events):
+    def _process_cv_events(self, cv_events):
         apis_to_be_tested = {}
 
         try:
@@ -489,8 +508,9 @@ class CloudvectorDAST(object):
                 else:
                     host_url = ""
                 api = \
-                str(event["http-req-url"]).lstrip("/").replace("/", "__").replace(" ", "").replace(host_url, "").split(
-                    "?")[0]
+                    str(event["http-req-url"]).lstrip("/").replace("/", "__").replace(" ", "").replace(host_url,
+                                                                                                       "").split(
+                        "?")[0]
                 if api not in apis_to_be_tested:
                     apis_to_be_tested[api] = {}
                 method = event["method"].lower()
@@ -571,6 +591,14 @@ class CloudvectorDAST(object):
         print("\n\ncreating Pytest test methods for fuzzing....\n")
         host_url, apis_to_be_tested = self._process_cv_events(cv_events)
 
+        if self.ceobj:
+            statefull = 1
+        else:
+            statefull = 0
+
+        if os.path.exists("tests/data"):
+            shutil.rmtree("tests/data")
+        shutil.copytree(os.path.join(templates_dir, "data"), "tests/data/")
         for type in fuzz_types:
             fuzzing_details = self._get_fuzz_details(type)
             with open(os.path.join(templates_dir, 'fuzz_test.j2')) as file_:
@@ -579,14 +607,16 @@ class CloudvectorDAST(object):
                 for method, info in v.items():
                     if not info["params"]:
                         continue
-                filename = str(k).replace("/", "_").replace("{", "").replace("}", "") + "_for_" + str(type) + "_fuzzing"
+                filename = str(k).replace("/", "_").replace("{", "").replace("}", "") + "_for_fuzzing"
                 if not os.path.exists("tests"):
                     os.mkdir("tests")
 
                 with open("tests/test_" + str(filename) + ".py", 'w+') as fh:
                     fh.write(template.render(apis_metadata=apis_to_be_tested[k], api_name=k, host_url=host_url,
-                                             fuzzing_details=fuzzing_details, fuzz_type=str(type)))
+                                             fuzzing_details=fuzzing_details, fuzz_type=str(type),
+                                             stateful_fuzz=statefull))
                 files_created.add("test_" + str(filename))
+
         print("\n\t\t......done creating pytest methods for fuzzing: " + str(files_created))
 
     def validate_pycode_for_syntax(self, code):
@@ -706,11 +736,321 @@ class CloudvectorDAST(object):
         print("\nNo. of events processed from Cloudvector Enterprise Dashboard: " + str(processed_events))
         return cv_requests
 
+    @staticmethod
+    def _parse_spaarc_report():
+        if not os.path.exists(os.path.join(os.getcwd(), 'spaarc_report.json')):
+            return
+        with open(os.path.join(os.getcwd(), 'spaarc_report.json')) as file_:
+            spaarc_info = json.load(file_).get("files")
+        spaarc_findings = {}
+        for file, api_info in spaarc_info.items():
+            for api_name, info in api_info.get("apis",{}).items():
+                api_name = api_name.split(":")[-1]
+                if api_name not in spaarc_findings:
+                    spaarc_findings[api_name] = info.get("violations",[])
+                else:
+                    spaarc_findings[api_name] += info.get("violations",[])
+        return spaarc_findings
+
+    @staticmethod
+    def generate_fuzz_report():
+        if not os.path.exists(os.path.join(os.getcwd(), 'tests/data/attack_info.json')):
+            print("\n\n\tThe flag '--generate-report' is a beta version only applicable for FUZZ tests at the moment!\n")
+            return
+        with open(os.path.join(os.getcwd(), 'tests/data/attack_info.json')) as file_:
+            attack_info = json.load(file_).get("info")
+
+        with open("report.json") as fobj:
+            report = json.load(fobj)
+
+        consolidated_report = {}
+        fuzz_metainfo = {}
+        fuzz_types = {"critical":{},"high":{},"medium":{},"low":{},"info":{},"bestpractice":{}}
+
+        def get_metadata_for_fuzzinput(in_file):
+            for type, subtype in attack_info.items():
+                for name, info in subtype.items():
+                    #print(name, info.get("file_set"), in_file, "\n\n\n")
+                    if str(in_file).lower() in [str(_).split("/")[-1].lower() for _ in info.get("file_set",[])]:
+                        return info
+            return None
+
+        for test in report["report"]["tests"]:
+            test_name = test["name"].split("::", 1)[-1]
+            api_method = test_name.split("test_")[-1].split("_")[0]
+            api_name = test_name.split(api_method, 1)[-1].split("for")[0].replace("__", "_").replace("_", "/")
+            stdout = test.get("call",{}).get("stdout")
+            #fuzz_type = test_name.split("for_")[-1].split("[")[0] + "_fuzz"
+            metadata = test.get("metadata")[0]
+            try:
+                response_length = metadata.split("::")[0]
+            except:
+                response_length = 0
+            try:
+                response_time = metadata.split("::")[1]
+            except:
+                response_time = 0
+            try:
+                response_code = metadata.split("::")[2]
+            except:
+                response_code = 0
+            try:
+                fuzz_sub_type = metadata.split("::")[3].split("/")[-1]
+            except:
+                fuzz_sub_type = "na"
+            try:
+                fuzz_type = metadata.split("::")[3].replace(fuzz_sub_type,"").replace("attack","").replace("//","/").split("/")[1]
+            except:
+                fuzz_type = "na"
+            try:
+                test_response = metadata.split("::")[4]
+            except:
+                test_response = None
+            try:
+                curl_command_to_try = stdout.split("CURL command to retry:")[-1].split("---------------------------------")[0].replace("\n","")
+            except:
+                curl_command_to_try = ""
+            if api_name not in consolidated_report:
+                consolidated_report[api_name] = {}
+            if api_method not in consolidated_report[api_name]:
+                consolidated_report[api_name][api_method] = {}
+
+            if fuzz_type not in consolidated_report[api_name][api_method]:
+                consolidated_report[api_name][api_method][fuzz_type] = {"count": 1, "status": [test["outcome"]],
+                                                                        "duration": [int(test["duration"])],
+                                                                        "resp_duration":[response_time],
+                                                                        "resp_length":[response_length],
+                                                                        "resp_code":[response_code]}
+            else:
+                consolidated_report[api_name][api_method][fuzz_type]["count"] += 1
+                consolidated_report[api_name][api_method][fuzz_type]["status"].append(test["outcome"])
+                consolidated_report[api_name][api_method][fuzz_type]["duration"].append(int(test["duration"]))
+                consolidated_report[api_name][api_method][fuzz_type]["resp_duration"].append(response_time)
+                consolidated_report[api_name][api_method][fuzz_type]["resp_length"].append(response_length)
+                consolidated_report[api_name][api_method][fuzz_type]["resp_code"].append(response_code)
+
+            fuzz_metadata = get_metadata_for_fuzzinput(fuzz_sub_type)
+
+            if fuzz_metadata:
+                fuzz_metainfo[fuzz_type] = {
+                        "description": fuzz_metadata.get("description"),
+                        "impact": fuzz_metadata.get("impact"),
+                        "severity": fuzz_metadata.get("severity"),
+                        "remedy": fuzz_metadata.get("remedy"),
+                        "classification": fuzz_metadata.get("classification"),
+                        "references": fuzz_metadata.get("references"),
+                        "ease_of_exploitation": fuzz_metadata.get("ease_of_exploitation")
+                    }
+
+
+            if str(test["outcome"]).lower() == "failed":
+                if not fuzz_metadata:
+                    continue
+                fuzz_severity = str(fuzz_metadata.get("severity")).lower()
+                fuzz_description = fuzz_metadata.get("description")
+                fuzz_impact = fuzz_metadata.get("impact")
+                fuzz_remedy = fuzz_metadata.get("remedy")
+                fuzz_classification = fuzz_metadata.get("classification")
+                fuzz_references = fuzz_metadata.get("references")
+                fuzz_ease_of_exploitation = fuzz_metadata.get("ease_of_exploitation")
+
+                if fuzz_type not in fuzz_types.get(fuzz_severity,{}):
+                    if str(fuzz_type):
+                        fuzz_types[fuzz_severity][fuzz_type] = {
+                            "description": fuzz_description,
+                            "severity": fuzz_severity,
+                            "impact": fuzz_impact,
+                            "remedy": fuzz_remedy,
+                            "classification": fuzz_classification,
+                            "references": fuzz_references,
+                            "ease_of_explanation": fuzz_ease_of_exploitation,
+                            "executions": [{
+                                                "api": api_name,
+                                                "api_method": api_method,
+                                                "api_response": test_response,
+                                                "req_curl": curl_command_to_try
+                                          }]
+                        }
+                else:
+                    fuzz_types[fuzz_severity][fuzz_type]["executions"].append({
+                                                "api": api_name,
+                                                "api_method": api_method,
+                                                "api_response": test_response,
+                                                "req_curl": curl_command_to_try
+                                          })
+
+
+        api_data = []
+        vulnerabilities_found = 0
+        total_api_calls = 0
+        for api, info in consolidated_report.items():
+            for method, fuzz_info in info.items():
+                total_tests = 0
+                success_tests = 0
+                failed_tests = 0
+                resp_length = []
+                resp_duration = []
+                resp_code = []
+                for type, info in fuzz_info.items():
+                    total_tests += len(info["status"])
+                    success_tests += info["status"].count("passed")
+                    failed_tests += info["status"].count("failed")
+                    resp_code += info["resp_code"]
+                    resp_duration += info["resp_duration"]
+                    resp_length += info["resp_length"]
+
+                vulnerabilities_found += failed_tests
+                record = {"name": api.replace("9i9","-"), "method": method, "resp_duration": resp_duration,
+                          "spec": "openapi_spec.json", "total_tests": total_tests,
+                          "success_tests": success_tests, "resp_length" : resp_length, "resp_code": resp_code,
+                          "failed_tests": failed_tests, "host": os.environ.get("CVIAST_TEST_APP_HOST",""), "app":"",
+                          "skipped_tests": int(total_tests)-int(success_tests)-int(failed_tests),
+                          "fuzz_type": type}
+                if record["failed_tests"] > 0:
+                    record["progress"] = 0
+                    api_data.insert(0, record)
+                else:
+                    record["progress"] = 1
+                    api_data.append(record)
+                total_api_calls+=record["total_tests"]
+
+        fuzz_data = {}
+        total_apis = 0
+
+        for api, info in consolidated_report.items():
+            for method, fuzz_info in info.items():
+                total_apis += 1
+                for type, info in fuzz_info.items():
+                    total_tests = 0
+                    success_tests = 0
+                    failed_tests = 0
+                    resp_length = []
+                    resp_duration = []
+                    resp_code = []
+                    total_tests += len(info["status"])
+                    success_tests += info["status"].count("passed")
+                    failed_tests += info["status"].count("failed")
+                    failed_apis = 0
+                    resp_length += info["resp_length"]
+                    resp_duration += info["resp_duration"]
+                    resp_code += info["resp_code"]
+                    if failed_tests > 0:
+                        failed_apis += 1
+                    if type not in fuzz_data:
+                        fuzz_data[type] = {"apis_tested": total_apis, "apis_failed": failed_apis,
+                                           "total_tests": total_tests,"success_tests": success_tests,
+                                           "failed_tests": failed_tests,"resp_length": resp_length,
+                                           "resp_duration":resp_duration, "resp_code":resp_code}
+                    else:
+                        fuzz_data[type]["apis_tested"] = total_apis
+                        fuzz_data[type]["apis_failed"] += failed_apis
+                        fuzz_data[type]["total_tests"] += total_tests
+                        fuzz_data[type]["success_tests"] += success_tests
+                        fuzz_data[type]["failed_tests"] += failed_tests
+                        fuzz_data[type]["resp_length"] += resp_length
+                        fuzz_data[type]["resp_duration"] += resp_duration
+                        fuzz_data[type]["resp_code"] += resp_code
+
+        fuzz_data_processed = []
+        for k, v in fuzz_data.items():
+            if len(k) == 0 or k == "na":
+                continue
+            if v["apis_failed"] > 0:
+                progress = 0
+                fuzz_data_processed.insert(0, {"progress": progress, "fuzz_type": k.replace("_fuzz", ""),
+                                               "total_apis": v["apis_tested"], "host": os.environ.get("CVIAST_TEST_APP_HOST",""),
+                                               "success_apis": v["apis_tested"] - v["apis_failed"],
+                                               "failed_apis": v["apis_failed"], "app":"",
+                                               "total_tests": v["total_tests"], "success_tests": v["success_tests"],
+                                               "failed_tests": v["failed_tests"],"resp_length":v["resp_length"],
+                                               "resp_duration":v["resp_duration"]})
+            else:
+                progress = 1
+                fuzz_data_processed.append(
+                    {"progress": progress, "fuzz_type": k.replace("_fuzz", ""), "total_apis": v["apis_tested"],
+                     "success_apis": v["apis_tested"] - v["apis_failed"], "failed_apis": v["apis_failed"],
+                     "total_tests": v["total_tests"], "success_tests": v["success_tests"], "app":"",
+                     "failed_tests": v["failed_tests"], "host": os.environ.get("CVIAST_TEST_APP_HOST",""),
+                     "resp_length":v["resp_length"],"resp_duration":v["resp_duration"],
+                     "skipped_tests": int(v["total_tests"])-int(v["success_tests"]-v["failed_tests"])})
+
+
+        with open(os.path.join(templates_dir, 'html/index.j2')) as file_:
+            template = Template(file_.read())
+
+        vulns_stats = {
+            "high": 0,
+            "low": 0,
+            "critical": 0,
+            "medium": 0,
+            "info": 0,
+            "bestpractice": 0,
+            "all": len(fuzz_types.get("medium", [])) + len(fuzz_types.get("high", [])) + len(fuzz_types.get("low", [])) + len(fuzz_types.get("critical", []))
+        }
+
+        spaarc_findings = CloudvectorDAST._parse_spaarc_report()
+        if spaarc_findings:
+            vulns_stats["info"]+= sum([len(_) for _ in list(spaarc_findings.values())])
+
+        for severity, types in fuzz_types.items():
+            for k, v in types.items():
+                vulns_stats[severity] = len(v.get("executions",[]))
+        with open("cviast_report.html", "w+") as file_:
+            file_.write(template.render(api_data=api_data, fuzz_data=fuzz_data_processed, total_api_calls=total_api_calls,
+                                        total_vulnerabilities=vulnerabilities_found, spaarc_findings=spaarc_findings,
+                                        test_time=datetime.datetime.fromtimestamp(time.time()).isoformat(), total_apis_scanned=total_apis,
+                                        fuzz_types=fuzz_types, vulns_stats=vulns_stats, fuzz_metadata=fuzz_metainfo,
+                                        test_lapse_time=str(round(int(report["report"]["summary"]["duration"])/60))+" min(s)"))
+        print("CVIAST report saved in " + str(os.getcwd()) + "/cviast_report.html")
+
+def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
+    """
+    Call in a loop to create terminal progress bar
+    @params:
+        iteration   - Required  : current iteration (Int)
+        total       - Required  : total iterations (Int)
+        prefix      - Optional  : prefix string (Str)
+        suffix      - Optional  : suffix string (Str)
+        decimals    - Optional  : positive number of decimals in percent complete (Int)
+        length      - Optional  : character length of bar (Int)
+        fill        - Optional  : bar fill character (Str)
+        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
+    """
+    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
+    filledLength = int(length * iteration // total)
+    bar = fill * filledLength + '-' * (length - filledLength)
+    print(f'\r{prefix} |{bar}| {percent}% {suffix}', end = printEnd)
+    # Print New Line on Complete
+    if iteration == total:
+        print()
+
 
 def main():
     import sys
     import getpass
     import yaml
+    # import argparse
+    # parser = argparse.ArgumentParser()
+    # parser.add_argument('--fuzz', help='To generate the fuzz test cases during test generation', action="store_true")
+    # parser.add_argument('--execute', help='To execute pytest tests, all CLI options to py test applicable', action="store_true")
+    # parser.add_argument('--generate-report', help='To generate report html for fuzz tests from an already executed suite using report.json', action="store_true")
+    # parser.add_argument('vars', nargs='*')
+    #
+    # args = parser.parse_args()
+
+    if sys.argv[1:]:
+        if sys.argv[1:][0] == "--generate-report":
+            CloudvectorDAST.generate_fuzz_report()
+            return
+        if sys.argv[1:][0] == "--execute":
+            arguments = sys.argv[2:]
+            arguments += ["--json=report.json", "--html=CVIAST-detailed_report.html", "--self-contained-html", "-vv"]
+            try:
+                pytest.main(arguments)
+                CloudvectorDAST.generate_fuzz_report()
+            except:
+                pass
+            return
     if os.path.exists(os.path.join(os.getcwd(), "my_cesetup.yaml")):
         with open(os.path.join(os.getcwd(), "my_cesetup.yaml")) as fobj:
             ce_details = yaml.load(fobj, Loader=yaml.FullLoader)
@@ -776,8 +1116,14 @@ def main():
                     str(cover_only_diff).lower(), input_params_file, enable_fuzzing)
     print(
         "\n Test generation complete! Making the pycode PEP complaint ( https://www.python.org/dev/peps/pep-0008/ ).....\n")
-    os.system("autopep8 --in-place --aggressive --aggressive tests/*.py")
-    print("\n\t\t..........done")
+    printProgressBar(0, len(os.listdir("tests")), prefix='Progress:', suffix='Complete', length=50)
+    i = 0
+    for _ in os.listdir("tests"):
+        if ".py" in _:
+            os.system("autopep8 --in-place --aggressive --aggressive tests/"+str(_))
+            printProgressBar(i + 1, len(os.listdir("tests")), prefix='Progress:', suffix='Complete', length=50)
+            i+=1
+    print("\n\n\t\t..........done")
 
 
 if __name__ == "__main__":
